#!/usr/bin/perl
$|=1;
use strict;

if ( $ENV{'HTTP_COOKIE'} eq "" )
{
	exit;
}

print "Content-Type: text/html\n\n";

my $hdr_name = "Name";
my $hdr_mac = "MAC Address";
my $hdr_ip = "IP Address (sender)";
my $hdr_pwd = "Password";
my $hdr_son = "SecureOn";
my $on = "on";
my $off = "off";
my $save = "Save";
my $edit = "Edit";
my $delete = "Delete";
my $confirm = "Do you really want to delete the device?";
my $err_name = "Please fill in a name.";
my $err_mac = "Invalid MAC Address. (Example: 1a:2b:3c:4d:5e:6f)";
my $err_ip = "Invalid IP address. (IPv4 or IPv6)";
my $err_net = "Invalid subnet. (IPv4/nn = 8->32)";
my $err_net6 = "Invalid prefix. (IPv6/nnn = 8->128)";
my $err_pwd = "Invalid password. (Max. 6 characters or 6 bytes hexadecimal)";
my $err_son = "SecureOn is only possible when the password is filled.";
my $err_msg = "";

if ( index($ENV{HTTP_ACCEPT_LANGUAGE},"nl") == 0  )
{
	$hdr_name = "Naam";
	$hdr_mac = "MAC adres";
	$hdr_ip = "IP adres (verzender)";
	$hdr_pwd = "Wachtwoord";
	$on = "aan";
	$off = "uit";
	$save = "Opslaan";
	$edit = "Wijzigen";
	$delete = "Verwijderen";
	$confirm = "Wil je het apparaat echt verwijderen?";
	$err_name = "De naam moet ingevuld worden.";
	$err_mac = "Ongeldig MAC adres. (Voorbeeld: 1a:2b:3c:4d:5e:6f)";
	$err_ip = "Ongeldig IP adres. (IPv4 of IPv6)";
	$err_net = "Ongeldig subnet. (IPv4/nn = 8->32)";
	$err_net6 = "Ongeldige prefix. (IPv6/nnn = 8->128)";
	$err_pwd = "Ongeldig wachtwoord. (Max. 6 karakters of 6 bytes hexadecimaal)";
	$err_son = "SecureOn is alleen mogelijk als het wachtwoord ingevuld is.";
}

my $WoLfilter = "../gateway/WOL-Gateway.conf";
my @dev_name = ();
my @dev_mac = ();
my @dev_ip = ();
my @dev_pwd = ();
my @dev_son = ();
my $act_row = 0;
my $sel_row = 0;
my $row = 0;

my $form_action = "";
my $form_row = 0;
my $form_name = "";
my $form_mac = "";
my $form_ip = "";
my $form_pwd = "";
my $form_son = "";
my $device = "";


if ( $ENV{'REQUEST_METHOD'} eq "POST" )
{
	my $form = "";
	my $pair = "";
	my $key = "";
	my $value = "";
	my %fields;
	read(STDIN,$form, $ENV{'CONTENT_LENGTH'});
	foreach $pair (split('&', $form))
	{
		if ($pair =~ /(.*)=(.*)/)
		{
			($key,$value) = ($1,$2);
			$value =~ s/\+/ /g;
			$value =~ s/%(..)/pack('c',hex($1))/eg;
			$fields{$key} = $value;
		}
	}
	$form_action = $fields{"action"};
	if ( $fields{"row"} eq "")
	{
		$form_row = -1;
	}
	else
	{
		$form_row = $fields{"row"};
	}
	$form_name = $fields{"name"};
	$form_mac = $fields{"mac"};
	$form_ip = $fields{"ip"};
	$form_pwd = $fields{"pwd"};
	$form_son = $fields{"son"};
	if (($form_action eq $save) || ($form_action eq $delete))
	{
		my $form_ip_type = 0;
		my $form_ip_sort = "";
		my $new_device = "";
		my $old_device = "";
		if ($form_action eq $save)
		{
			# Validate the input
			if (($form_son eq "true") && ($form_pwd eq ""))
			{
				$err_msg = $err_son;
			}
			if ((($form_pwd !~ /[\da-f]{2}[\da-f]{2}[\da-f]{2}[\da-f]{2}[\da-f]{2}[\da-f]{2}/i) && length($form_pwd) > 6))
			{
				$err_msg = $err_pwd;
			}
			if ($form_ip ne "")
			{
				if ( $form_ip =~ m/^(\d\d?\d?)\.(\d\d?\d?)\.(\d\d?\d?)\.(\d\d?\d?)$/ )
				{
					if ($1 > 255 || $2 > 255 || $3 > 255 || $4 > 255)
					{
						$err_msg = $err_ip;
					}
					else
					{
						$form_ip_type = 4;
						$form_ip_sort = sprintf("%03d.%03d.%03d.%03d", split(/\./, $form_ip));
					}
				}
				else
				{
					if ( $form_ip =~ m/^(\d\d?\d?)\.(\d\d?\d?)\.(\d\d?\d?)\.(\d\d?\d?)\/(\d\d?)$/ )
					{
						if($1 > 255 || $2 > 255 || $3 > 255 || $4 > 255 || $5 < 8 || $5 > 32)
						{
							$err_msg = $err_net;
						}
						else
						{
							$form_ip_type = 4;
							$form_ip_sort = sprintf("%03d.%03d.%03d.%03d/%02d", split(/\.|\//, $form_ip));
						}
					}
					else
					{
					  	if ( $form_ip !~/^\s*((([0-9A-Fa-f]{1,4}:){7}([0-9A-Fa-f]{1,4}|:))|(([0-9A-Fa-f]{1,4}:){6}(:[0-9A-Fa-f]{1,4}|((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3})|:))|(([0-9A-Fa-f]{1,4}:){5}(((:[0-9A-Fa-f]{1,4}){1,2})|:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3})|:))|(([0-9A-Fa-f]{1,4}:){4}(((:[0-9A-Fa-f]{1,4}){1,3})|((:[0-9A-Fa-f]{1,4})?:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){3}(((:[0-9A-Fa-f]{1,4}){1,4})|((:[0-9A-Fa-f]{1,4}){0,2}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){2}(((:[0-9A-Fa-f]{1,4}){1,5})|((:[0-9A-Fa-f]{1,4}){0,3}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){1}(((:[0-9A-Fa-f]{1,4}){1,6})|((:[0-9A-Fa-f]{1,4}){0,4}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(:(((:[0-9A-Fa-f]{1,4}){1,7})|((:[0-9A-Fa-f]{1,4}){0,5}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:)))(%.+)?\s*$/ )
						{
							if ( $form_ip !~/^\s*((([0-9A-Fa-f]{1,4}:){7}([0-9A-Fa-f]{1,4}|:))|(([0-9A-Fa-f]{1,4}:){6}(:[0-9A-Fa-f]{1,4}|((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3})|:))|(([0-9A-Fa-f]{1,4}:){5}(((:[0-9A-Fa-f]{1,4}){1,2})|:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3})|:))|(([0-9A-Fa-f]{1,4}:){4}(((:[0-9A-Fa-f]{1,4}){1,3})|((:[0-9A-Fa-f]{1,4})?:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){3}(((:[0-9A-Fa-f]{1,4}){1,4})|((:[0-9A-Fa-f]{1,4}){0,2}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){2}(((:[0-9A-Fa-f]{1,4}){1,5})|((:[0-9A-Fa-f]{1,4}){0,3}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){1}(((:[0-9A-Fa-f]{1,4}){1,6})|((:[0-9A-Fa-f]{1,4}){0,4}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(:(((:[0-9A-Fa-f]{1,4}){1,7})|((:[0-9A-Fa-f]{1,4}){0,5}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:)))(%.+)?\/(\d\d?\d?)\s*$/ )
							{
								$err_msg = $err_ip;
							}
							else
							{
								my @spl = split("\/", $form_ip);
								my ($IPv6_Address, $IPv6_Prefix) = @spl[0,1];
								if ((int($IPv6_Prefix) < 8) || (int($IPv6_Prefix) > 128))
								{
									$err_msg = $err_net6;
								}
								else
								{
									$form_ip_type = 6;
									$form_ip_sort = $form_ip;
								}
							}
						}
						else
						{
							$form_ip_type = 6;
							$form_ip_sort = $form_ip;
						}
					}
				}
			}
			if ($form_mac !~ /[\da-f]{2}[:\-][\da-f]{2}[:\-][\da-f]{2}[:\-][\da-f]{2}[:\-][\da-f]{2}[:\-][\da-f]{2}/i)
			{
				$err_msg = $err_mac;
			}
			if ($form_name eq "")
			{
				$err_msg = $err_name;
			}
			if ($err_msg eq "")
			{
				$form_mac =~ s/-/:/g;
				$new_device = $form_name . "," . $form_mac . "," . $form_ip . "," . $form_pwd . "," . $form_son;
			}
		}
		if ($err_msg eq "")
		{
			my $saved = 0;
			if (-e $WoLfilter)
			{
				open(WoL,$WoLfilter);
				while(my $line = <WoL>)
				{
					chomp($line);
					(@dev_name[$row], @dev_mac[$row], @dev_ip[$row], @dev_pwd[$row], @dev_son[$row]) = split(',', $line, 5);
					$row ++;
				}
				close WoL;
			}
			my $rows = $row;
			open(WoL, '>', $WoLfilter) or die $!;
			if ($rows > 0)
			{
				$row = 0;
				while ($row < $rows)
				{
					my $dev_ip_type = 0;
					my $dev_ip_sort = "";
					if ( $dev_ip[$row] ne "" )
					{
						if ( $dev_ip[$row] =~ m/^(\d\d?\d?)\.(\d\d?\d?)\.(\d\d?\d?)\.(\d\d?\d?)$/ )
						{
							$dev_ip_type = 4;
							$dev_ip_sort = sprintf("%03d.%03d.%03d.%03d", split(/\./, $dev_ip[$row]));
						}
						else
						{
							if ( $dev_ip[$row] =~ m/^(\d\d?\d?)\.(\d\d?\d?)\.(\d\d?\d?)\.(\d\d?\d?)\/(\d\d?)$/ )
							{
								$dev_ip_type = 4;
								$dev_ip_sort = sprintf("%03d.%03d.%03d.%03d/%02d", split(/\.|\//, $dev_ip[$row]));
							}
							else
							{
								$dev_ip_type = 6;
								$dev_ip_sort = $dev_ip[$row];
							}
						}
					}
					$sel_row = $row + 1;
					if ($form_row != $sel_row)
					{
						$old_device = $dev_name[$row] . "," . $dev_mac[$row] . "," . $dev_ip[$row] . "," . $dev_pwd[$row] . "," . $dev_son[$row];
						if (($new_device ne "") && ($saved == 0))
						{
							if ((lc($form_name) lt lc($dev_name[$row])) || ((lc($form_name) eq lc($dev_name[$row])) && (lc($form_mac) lt lc($dev_mac[$row]))) || ((lc($form_name) eq lc($dev_name[$row])) && (lc($form_mac) eq lc($dev_mac[$row])) && (($form_ip_type < $dev_ip_type) || (($form_ip_type == $dev_ip_type) && ($form_ip_sort lt $dev_ip_sort)))) || ((lc($form_name) eq lc($dev_name[$row])) && (lc($form_mac) eq lc($dev_mac[$row])) && (($form_ip_type == $dev_ip_type) && ($form_ip_sort eq $dev_ip_sort)) && (lc($form_pwd) lt lc($dev_pwd[$row]))) || ((lc($form_name) eq lc($dev_name[$row])) && (lc($form_mac) eq lc($dev_mac[$row])) && (($form_ip_type == $dev_ip_type) && ($form_ip_sort eq $dev_ip_sort)) && (lc($form_pwd) eq lc($dev_pwd[$row])) && (lc($form_son) lt lc($dev_son[$row]))))
							{
								print WoL "$new_device\n";
								$saved = 1;
							}
						}
						print WoL "$old_device\n";
					}
					$row ++;
				}
			}
			if ($new_device ne "")
			{
				if ($saved == 0)
				{
					print WoL "$new_device\n";
				}
			}
			close WoL;
			$act_row = 0;
		}
		else
		{
			$act_row = $form_row;
		}
	}
	else
	{
		$form_row = $form_action;
		$act_row = $form_row;
	}
}

# Load the WoL filter
$row = 0;
if (-e $WoLfilter)
{
	open(WoL,$WoLfilter);
	while(my $line = <WoL>)
	{
		chomp($line);
		(@dev_name[$row], @dev_mac[$row], @dev_ip[$row], @dev_pwd[$row], @dev_son[$row]) = split(',', $line, 5);
		$row ++;
	}
	close(WoL);
}
my $rows = $row;

print "<!DOCTYPE html>";
print "<html class=\"img-no-display\">";

if (-e "index.head")
{
	open HTML, "index.head" or die $!;
	while (<HTML>)
	{
		print;
	}
	close HTML;
}

print "<body>";
print "<form id=\"WakeOnLAN\" name=\"WakeOnLAN\" method=\"post\" action=\"index.cgi\">\n";
if (-e "index.js")
{
	open HTML, "index.js" or die $!;
	while (<HTML>)
	{
		print;
	}
	close HTML;
}
print "<div style=\"margin-left:0px;margin-right:15px\"><div style=\"margin-right:15px\"><table>\n<tr>";
print "<th style=\"border-left-style:hidden;border-right-style:hidden\" width=\"54px\" align=\"center\"><a href=\"help.cgi\"><img src=\"images/info.png\" border=\"0\" align=\"center\" height=24 width=54></img></a></th>";
print "<th style=\"border-left-style:hidden;border-right-style:hidden\" width=\"40%\" align=\"left\">$hdr_name</th>";
print "<th style=\"border-left-style:hidden;border-right-style:hidden\" width=\"20%\" align=\"left\">$hdr_mac</th>";
print "<th style=\"border-left-style:hidden;border-right-style:hidden\" width=\"20%\" align=\"left\">$hdr_ip</th>";
print "<th style=\"border-left-style:hidden;border-right-style:hidden\" width=\"10%\" align=\"left\">$hdr_pwd</th>";
print "<th style=\"border-left-style:hidden;border-right-style:hidden\" width=\"10%\" align=\"center\">$hdr_son</th>";
print "</tr>";

print "<form id=\"WakeOnLAN\" name=\"WakeOnLAN\" method=\"post\" action=\"index.cgi\">";
if ($rows > 0)
{
	$row = 0;
	while ($row < $rows)
	{
		$sel_row = $row + 1;
		print "\n<tr>";
		if ($act_row == $sel_row)
		{
			print "<td style=\"border-left-style:hidden;border-right-style:hidden\" width=\"54px\" align=\"center\"><input type=\"hidden\" name=\"row\" value=\"$sel_row\"><img src=\"images/edit.png\" align=\"center\" height=16 width=16></img></td>";
			if ($err_msg ne "")
			{
				print "<td style=\"border-left-style:hidden;border-right-style:hidden\" width=\"40%\" align=\"left\"><input style=\"width:100%\" type=\"text\" name=\"name\" value=\"$form_name\"></td>";
				print "<td style=\"border-left-style:hidden;border-right-style:hidden\" width=\"20%\" align=\"left\"><input style=\"width:100%\" type=\"text\" name=\"mac\" maxlength=\"17\" value=\"$form_mac\"></td>";
				print "<td style=\"border-left-style:hidden;border-right-style:hidden\" width=\"20%\" align=\"left\"><input style=\"width:100%\" type=\"text\" name=\"ip\" maxlength=\"39\" value=\"$form_ip\"></td>";
				print "<td style=\"border-left-style:hidden;border-right-style:hidden\" width=\"10%\" align=\"left\"><input style=\"width:100%\" type=\"text\" name=\"pwd\" maxlength=\"12\" value=\"$form_pwd\"></td>";
				if ($form_son ne "true")
				{
					print "<td style=\"border-left-style:hidden;border-right-style:hidden\" width=\"10%\" align=\"center\"><input type=\"checkbox\" name=\"son\" value=\"true\"></td>";
				}
				else
				{
					print "<td style=\"border-left-style:hidden;border-right-style:hidden\" width=\"10%\" align=\"center\"><input type=\"checkbox\" name=\"son\" value=\"true\" checked></td>";
				}
			}
			else
			{
				print "<td style=\"border-left-style:hidden;border-right-style:hidden\" width=\"40%\" align=\"left\"><input style=\"width:100%\" type=\"text\" name=\"name\" value=\"$dev_name[$row]\"></td>";
				print "<td style=\"border-left-style:hidden;border-right-style:hidden\" width=\"20%\" align=\"left\"><input style=\"width:100%\" type=\"text\" name=\"mac\" maxlength=\"17\" value=\"$dev_mac[$row]\"></td>";
				print "<td style=\"border-left-style:hidden;border-right-style:hidden\" width=\"20%\" align=\"left\"><input style=\"width:100%\" type=\"text\" name=\"ip\" maxlength=\"39\" value=\"$dev_ip[$row]\"></td>";
				print "<td style=\"border-left-style:hidden;border-right-style:hidden\" width=\"10%\" align=\"left\"><input style=\"width:100%\" type=\"text\" name=\"pwd\" maxlength=\"12\" value=\"$dev_pwd[$row]\"></td>";
				if (($dev_pwd[$row] eq "") || ($dev_son[$row] ne "true"))
				{
					print "<td style=\"border-left-style:hidden;border-right-style:hidden\" width=\"10%\" align=\"center\"><input type=\"checkbox\" name=\"son\" value=\"true\"></td>";
				}
				else
				{
					print "<td style=\"border-left-style:hidden;border-right-style:hidden\" width=\"10%\" align=\"center\"><input type=\"checkbox\" name=\"son\" value=\"true\" checked></td>";
				}
			}
		}
		else
		{
			print "<td style=\"border-left-style:hidden;border-right-style:hidden\" width=\"54px\" align=\"center\"><button type=\"submit\" name=\"action\" value=\"$sel_row\"><img src=\"images/edit.png\" align=\"middle\" valign=\"middle\" style=\"cursor: pointer;\"></button></td>";
			print "<td style=\"border-left-style:hidden;border-right-style:hidden\" width=\"40%\" align=\"left\">$dev_name[$row]</td>";
			print "<td style=\"border-left-style:hidden;border-right-style:hidden\" width=\"20%\" align=\"left\">$dev_mac[$row]</td>";
			print "<td style=\"border-left-style:hidden;border-right-style:hidden\" width=\"20%\" align=\"left\">$dev_ip[$row]</td>";
			print "<td style=\"border-left-style:hidden;border-right-style:hidden\" width=\"10%\" align=\"left\">$dev_pwd[$row]</td>";
			if ( $dev_pwd[$row] ne "")
			{
				if ( $dev_son[$row] eq "true" )
				{
					print "<td style=\"border-left-style:hidden;border-right-style:hidden\" width=\"10%\" align=\"center\">$on</td>";
				}
				else
				{
					print "<td style=\"border-left-style:hidden;border-right-style:hidden\" width=\"10%\" align=\"center\">$off</td>";
				}
			}
			else
			{
				print "<td style=\"border-left-style:hidden;border-right-style:hidden\" width=\"10%\" align=\"center\"></td>";
			}
		}
		print "</tr>";
		$row ++;
	}
}

print "\n<tr>";
if ( $act_row == 0 )
{
	print "<td style=\"border-left-style:hidden;border-right-style:hidden\" width=\"54px\" align=\"center\"><input type=\"hidden\" name=\"row\" value=\"0\"><img src=\"images/edit.png\" align=\"center\" height=16 width=16></td>";
	if ( $err_msg ne "" )
	{
		print "<td style=\"border-left-style:hidden;border-right-style:hidden\" width=\"40%\" align=\"left\"><input style=\"width:100%\" type=\"text\" name=\"name\" value=\"$form_name\"></td>";
		print "<td style=\"border-left-style:hidden;border-right-style:hidden\" width=\"20%\" align=\"left\"><input style=\"width:100%\" type=\"text\" name=\"mac\" maxlength=\"17\" value=\"$form_mac\"></td>";
		print "<td style=\"border-left-style:hidden;border-right-style:hidden\" width=\"20%\" align=\"left\"><input style=\"width:100%\" type=\"text\" name=\"ip\" maxlength=\"39\" value=\"$form_ip\"></td>";
		print "<td style=\"border-left-style:hidden;border-right-style:hidden\" width=\"10%\" align=\"left\"><input style=\"width:100%\" type=\"text\" name=\"pwd\" maxlength=\"12\" value=\"$form_pwd\"></td>";
		if ($form_son ne "true")
		{
			print "<td style=\"border-left-style:hidden;border-right-style:hidden\" width=\"10%\" align=\"center\"><input type=\"checkbox\" name=\"son\" value=\"true\"></td>";
		}
		else
		{
			print "<td style=\"border-left-style:hidden;border-right-style:hidden\" width=\"10%\" align=\"center\"><input type=\"checkbox\" name=\"son\" value=\"true\" checked></td>";
		}
	}
	else
	{
		print "<td style=\"border-left-style:hidden;border-right-style:hidden\" width=\"40%\" align=\"left\"><input style=\"width:100%\" type=\"text\" name=\"name\"></td>";
		print "<td style=\"border-left-style:hidden;border-right-style:hidden\" width=\"20%\" align=\"left\"><input style=\"width:100%\" type=\"text\" name=\"mac\" maxlength=\"17\"></td>";
		print "<td style=\"border-left-style:hidden;border-right-style:hidden\" width=\"20%\" align=\"left\"><input style=\"width:100%\" type=\"text\" name=\"ip\"  maxlength=\"39\"></td>";
		print "<td style=\"border-left-style:hidden;border-right-style:hidden\" width=\"10%\" align=\"left\"><input style=\"width:100%\" type=\"text\" name=\"pwd\" maxlength=\"12\"></td>";
		print "<td style=\"border-left-style:hidden;border-right-style:hidden\" width=\"10%\" align=\"center\"><input type=\"checkbox\" name=\"son\" value=\"true\"></td>";
	}
}
else
{
	print "<td style=\"border-left-style:hidden;border-right-style:hidden\" width=\"54px\" align=\"center\"><button type=\"submit\" name=\"action\" value=\"0\"> <img src=\"images/edit.png\" align=\"middle\" valign=\"middle\" style=\"cursor: pointer;\"></button></td>";
	print "<td style=\"border-left-style:hidden;border-right-style:hidden\" width=\"40%\" align=\"left\"></td>";
	print "<td style=\"border-left-style:hidden;border-right-style:hidden\" width=\"20%\" align=\"left\"></td>";
	print "<td style=\"border-left-style:hidden;border-right-style:hidden\" width=\"20%\" align=\"left\"></td>";
	print "<td style=\"border-left-style:hidden;border-right-style:hidden\" width=\"10%\" align=\"left\"></td>";
	print "<td style=\"border-left-style:hidden;border-right-style:hidden\" width=\"10%\" align=\"center\"></td>";
}
print "</tr>";

print "\n</table></div></div>";
print "<div align=\"center\"><p><input type=\"submit\" id=\"save\" name=\"action\" value=\"$save\" onmouseover=\"\" style=\"cursor: pointer;\">";
if ( $act_row != 0)
{
	print "<input type=\"submit\" onClick=\"return confirm(\'$confirm\')\" id=\"delete\" name=\"action\" value=\"$delete\" onmouseover=\"\" style=\"cursor: pointer;\">";
}
print "</p></div>";
print "</form>";

if ( $err_msg ne "" )
{
	print "<div align=\"center\"><p><font color=\"red\">$err_msg</font></p></div>";
}
print "</body></html>";
